@@ -3,7 +3,7 @@
    Copyright (C) 2003, 2004, 2006 Peter Palfrader
    Copyright (C) 2008, 2009 Carsten Hey
 
-   $Id: pkginfo.c 763 2009-02-19 11:49:00Z carsten-guest $
+   $Id: pkginfo.c 875 2011-03-19 22:51:12Z carsten $
 
    Distributed under the terms of the Artistic License.
 */
@@ -64,7 +64,7 @@ init_pkg_regex()
 	if (guess_chk(GUESS_DEV))
 	    strcat(guess, "-dev$|");
 	if (guess_chk(GUESS_DEBUG))
-	    strcat(guess, "-dbg$|");
+	    strcat(guess, "-dbg(|sym)$|");
 	if (guess_chk(GUESS_COMMON))
 	    strcat(guess, "-common$|");
 	if (guess_chk(GUESS_DATA))
@@ -74,7 +74,7 @@ init_pkg_regex()
 	if (guess_chk(GUESS_KERNEL))
 	    strcat(guess, "(-modules|^nvidia-kernel)-.*[[:digit:]]+\\.[[:digit:]]+\\.[[:digit:]]+");
 	if (guess_chk(GUESS_SECTION)) {
-	    regcomp(&re_gnugrepv, "(-perl|-dev|-doc|-dbg)$", 
+	    regcomp(&re_gnugrepv, "(-perl|-dev|-doc|-dbg)$|^lib(mono|pam|reoffice)|-(ruby[[:digit:].]*|bin|tools|utils|dbgsym)$", 
 		    REG_EXTENDED | REG_FLAGS);
 	}
 
@@ -87,7 +87,7 @@ init_pkg_regex()
 	if (guess_chk(GUESS_DUMMY)) {
 	    regcomp(&re_descdummy, "^Description:.*dummy", 
 		    REG_EXTENDED | REG_FLAGS);
-	    regcomp(&re_desctransit, "^Description:.*transition", 
+	    regcomp(&re_desctransit, "^Description:.*transition(|n)($|ing|al|ary| package| purposes)", 
 		    REG_EXTENDED | REG_FLAGS);
 	}
 	if (guess[strlen(guess)-1] == '|')
@@ -284,7 +284,10 @@ get_pkg_provides(const char *line, pkg_info * package)
 void
 get_pkg_name(const char *line, pkg_info * package)
 {
-    (void) set_dep(&(package->self), strchr(line, ':') + 1);
+    char *name = strchr(line, ':') + 1;
+
+    package->self.name = xstrdup(name);
+    package->self.namehash = strhash(name);
 }
 
 void
