@@ -11,7 +11,7 @@
 # Copyright (c) 2000 Goswin Brederlow <goswin.brederlow@student.uni-tuebingen.de>
 # Copyright (c) 2000, 2003, 2004, 2005, 2006 Peter Palfrader <peter@palfrader.org>
 # Copyright (c) 2003, 2004, 2007, 2008 Jörg Sommer <joerg@alea.gnuu.de>
-# Copyright (c) 2008 Carsten Hey <c.hey@web.de>
+# Copyright (c) 2008, 2009 Carsten Hey <c.hey@web.de>
 #
 # This program is dual licensed either unter the GNU GPL as published
 # by the Free Software Foundation; either version 2, or (at your option)
@@ -23,22 +23,24 @@
 # Debian systems. If you have problems obtaining the files please
 # write to the authors.
 #
-# $Id: orphaner.sh 730 2008-09-12 19:47:14Z carsten-guest $
+# $Id: orphaner.sh 874 2011-03-19 22:50:28Z carsten $
 
 set -e
 
 
 
-REVISION='$Rev: 730 $';
+REVISION='$Rev: 874 $';
 VERSION="(r${REVISION#\$Rev: }"
 VERSION="${VERSION% \$})"
 
 OPTIONS=$@
 VALIDOPTIONS='^-([aDHns]|-libdevel|-guess-(.+)|-find-(.+)|-nice-mode|-all-packages|-priority(.+)|p(.+)|-show-section|-force-hold)[[:space:]]$'
 VALIDKEEPOPTIONS='^-([aDHns]|-libdevel|-guess-(.+)|-find-(.+)|-nice-mode|-all-packages|-priority(.+)|p(.+)|-show-section|-force-hold)[[:space:]]$'
+SKIPAPT=0
+CIRCULAR=0
 
 # LC_COLLATE=pl_PL or similar breaks orphaner under some circumstances, see
-# Debian bug #495818
+# Debian bug #495818.
 export LC_COLLATE=C
 
 if which gettext > /dev/null; then
@@ -52,7 +54,7 @@ fi
 TEXTDOMAIN=deborphan
 export TEXTDOMAIN
 # xgettext:sh-format
-USAGE=$(gettext 'Usage: %s [--help|--purge] [deborphan options]')'\n'
+USAGE=$(gettext 'Usage: %s [--help|--purge|--skip-apt] [deborphan options]')'\n'
 # xgettext:no-sh-format
 SEE_ORPHANER=$(gettext 'See orphaner(8) and deborphan(1) for a list of valid options.')
 # xgettext:sh-format
@@ -72,11 +74,12 @@ ORPHANER_INSTRUCTION=$(gettext 'Select packages for removal or cancel to quit:')
 # xgettext:no-sh-format
 NO_ORPHANS_FOUND=$(gettext 'No orphaned packages found.')
 # xgettext:no-sh-format
-DEBORPHAN_REMOVED=$(gettext 'deborphan got removed.  Exiting.')
+DEBORPHAN_REMOVED=$(gettext '"deborphan" got removed.  Exiting.')
 # xgettext:no-sh-format
-APT_GET_REMOVED=$(gettext 'apt got removed.  Exiting.')
+APT_GET_REMOVED=$(gettext '"apt" got removed.  Exiting.')
 # xgettext:no-sh-format
-APT_GET_LOCKFAIL=$(gettext 'apt is not installed, broken dependencies found or could not open lock file, are you root?  Printing apt-get commandline and exiting:')
+APT_GET_LOCKFAIL=$(gettext '"apt" is not installed, broken dependencies found or could not open lock file, are you root?  Printing "apt-get" commandline and exiting:')
+SKIPAPT_SET=$(gettext 'Explicitly specified status file or requested calling "apt-get" to be skipped.  Printing "apt-get" commandline and exiting:')
 # xgettext:no-sh-format
 REMOVING=$(gettext 'Removing %s')'\n'
 
@@ -87,7 +90,7 @@ APT_GET_ERROR=$(gettext '"apt-get" returned with exitcode %s.')'\n'
 # xgettext:sh-format
 DIALOG_ERROR=$(gettext '"dialog" returned with exitcode %s.')'\n'
 # xgettext:no-sh-format
-NUMBER_OF_PACKAGES_ERROR=$(gettext '"apt-get" tries to remove more packages than requested by orphaner.  Exiting.')'\n'
+NUMBER_OF_PACKAGES_ERROR=$(gettext '"apt-get" tries to remove more packages than requested by "orphaner".  Exiting.')'\n'
 
 # xgettext:no-sh-format
 SIMULATE_BUTTON=$(gettext 'Simulate')
@@ -95,6 +98,8 @@ SIMULATE_BUTTON=$(gettext 'Simulate')
 # xgettext:no-sh-format
 PRESS_ENTER_TO_CONTINUE=$(gettext 'Press enter to continue.')
 
+# xgettext:no-sh-format
+CIRCULAR_NOT_SUPPORTED=$(gettext '"find-circular" is currently not supported by "orphaner".  Exiting.')
 
 if ! which dialog >/dev/null ; then
 	printf "$MISSING_DIALOG" $0 >&2
@@ -137,10 +142,7 @@ else
 		fi
 	}
 
-	case "${BASH_VERSION:+bash}" in
-		bash) trap sigwinch_handle SIGWINCH;;
-		*) trap sigwinch_handle 20 28;;
-	esac
+	trap sigwinch_handle WINCH
 fi
 
 if [ $lines -lt 12 -o $columns -lt 50 ]; then
@@ -218,7 +220,7 @@ keeping_list() { #{{{
 } #}}}
 
 deborphan_list() { #{{{
-	{ deborphan -s $@ || echo "ERROR"; } \
+	{ deborphan -e diff,mktemp -s $@ || echo "ERROR"; } \
 		| while read SECTION PACKAGE; do
 		echo $PACKAGE $SECTION off
 	done
@@ -257,7 +259,7 @@ doorphans() { #{{{
 '
 	while true; do
 		OLD_ORPHANED="$ORPHANED"
-		ORPHANED=$(deborphan_list $OPTIONS ${EXCLUDE:+--exclude=$EXCLUDE}| LANG=C sort)
+		ORPHANED=$(deborphan_list $OPTIONS ${EXCLUDE:+--exclude=$EXCLUDE} | sort)
 		if [ "$ORPHANED" = "ERROR off" ] ; then
 			echo $DEBORPHAN_ERROR >&2
 			exit 1
@@ -398,7 +400,7 @@ __OORPH_EOT
 					PACKAGES_TO_REMOVE="${PACKAGES_TO_REMOVE% }"
 #					printf "$REMOVING" "$PACKAGES_TO_REMOVE"
 					APT_GET_CMDLN="apt-get $PURGE --show-upgraded --assume-yes remove $PACKAGES_TO_REMOVE"
-					if apt-get check -q -q; then
+					if apt-get check -q -q 2> /dev/null && [ $SKIPAPT -eq 0 ]; then
 						LC_ALL=C $APT_GET_CMDLN --simulate \
 						| grep -q "^0 upgraded, 0 newly installed, `echo "$PACKAGES_TO_REMOVE" | wc -w` to remove and"
 						if [ $? -ne 0 ]; then
@@ -408,8 +410,12 @@ __OORPH_EOT
 						fi
 						$APT_GET_CMDLN || ERROR=$?
 					else
-						printf '%s\n' "$APT_GET_LOCKFAIL" >&2
-						printf '%s\n' "$APT_GET_CMDLN" >&2
+						if [ $SKIPAPT -eq 0 ]; then
+							printf '%s\n' "$APT_GET_LOCKFAIL" >&2
+						else
+							printf '%s\n' "$SKIPAPT_SET" >&2
+						fi
+						printf '%s\n' "$APT_GET_CMDLN"
 						exit 1
 					fi
 					unset APT_GET_CMDLN PACKAGES_TO_REMOVE
@@ -504,7 +510,7 @@ __EOT
 						--textbox $TMPFILE $BOXSIZE
 					;; #}}}
 				3) # Simulate-Button #{{{
-					EXCLUDE=$(while read pkg; do printf $pkg,; done < $TMPFILE)
+					EXCLUDE=$(grep -v '^----$' $TMPFILE | while read pkg; do printf $pkg,; done)
 					EXCLUDE=${EXCLUDE%,}
 					break
 					;; #}}}
@@ -517,7 +523,7 @@ __EOT
 	done #}}}
 } #}}}
 
-# purge set?
+# parse options # {{{
 case " $OPTIONS " in
 	*" --purge "*)
 		OPTIONS="${OPTIONS%%--purge*}${OPTIONS#*--purge}"
@@ -525,6 +531,37 @@ case " $OPTIONS " in
 		;;
 esac
 
+case " $OPTIONS " in
+	*" -f "*)
+		SKIPAPT=1
+		;;
+	*" --status-file"*)
+		SKIPAPT=1
+		;;
+esac
+
+case " $OPTIONS " in
+	*" -c "*)
+		CIRCULAR=1
+		;;
+	*" --find-circular"*)
+		CIRCULAR=1
+		;;
+esac
+
+case " $OPTIONS " in
+	*" --skip-apt "*)
+		OPTIONS="${OPTIONS%%--skip-apt*}${OPTIONS#*--skip-apt}"
+		SKIPAPT=1
+		;;
+esac
+# }}}
+
+if [ $CIRCULAR -eq 1 ]; then
+	printf '%s\n' "$CIRCULAR_NOT_SUPPORTED" >&2
+	exit 1
+fi
+
 case $0 in
 	*orphaner|*orphaner.sh) doorphans;;
 	*editkeep) editkeepers;;
